@@ -1,17 +1,15 @@
-;;; -*- Emacs-Lisp -*-
-;;; YaTeX add-in function generator.
-;;; yatexgen.el rev.2
-;;; (c )1991-1994 by HIROSE Yuuji.[yuuji@ae.keio.ac.jp]
-;;; Last modified Fri Jul  8 00:46:09 1994 on figaro
-;;; $Id: yatexgen.el,v 1.50 1994/07/07 16:37:24 yuuji Rel $
+;;; yatexgen.el --- YaTeX add-in function generator(rev.5)
 
+;;; (c)1991-1995,1999,2000,2017 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Sun Dec 21 14:04:49 2014 on firestorm
+;;; $Id$
+
+;;; Code:
 (require 'yatex)
-(provide 'yatexgen)
 
 (defmacro YaTeX-setq (var japanese english)
   (list 'setq var
-	(if YaTeX-japan japanese english))
-)
+	(if YaTeX-japan japanese english)))
 
 (put 'YaTeX-setq 'lisp-indent-hook 1)
 
@@ -30,8 +28,7 @@ following my messages.  Then, at making actual function, operate
 reading my messages *carefully*, or you'll fail to generate appropriate
 function.
 
-  Hit return key!"
-)
+  Hit return key!")
 
 (YaTeX-setq YaTeX-generate-start-message
   "͂߂.\n1.o^⊮Ă݂.
@@ -41,33 +38,27 @@ function.
   "Let's begin completion for which you want to make add-in function.
 If you want to make add-in function for \\documentstyle input only
 `\\documentstyle{}' *with* completion of yatex-mode.
-If you finish this, please press RET."
-)
+If you finish this, please press RET.")
 
 (YaTeX-setq YaTeX-generate-abort-message
   "߂A߂`߂ǂ`"
-  "Aborted."
-)
+  "Aborted.")
 
 (YaTeX-setq YaTeX-generate-same-message
   "ꂶAȂɂςĂ˂[! ߂B"
-  "I found no difference between them.  So I'm quitting."
-)
+  "I found no difference between them.  So I'm quitting.")
 
 (YaTeX-setq YaTeX-generate-invalid-message
   "́AƖƂ̂."
-  "It's impossible."
-)
+  "It's impossible.")
 
 (YaTeX-setq YaTeX-generate-idontknow-message
   "`AĂ悭킩ȂȂB΂ł߂˃F`"
-  "Sorry I can't tell your adding method."
-)
+  "Sorry I can't tell your adding method.")
 
 (YaTeX-setq YaTeX-generate-confirm-message
   "ƂƂ́At͂ł̂"
-  "Is it additional string of add-in function?"
-)
+  "Is it additional string of add-in function?")
 
 (YaTeX-setq YaTeX-generate-output-message
   "2.Aɂ̂ *J[\̈ʒu* Ă݂. 
@@ -76,8 +67,7 @@ If you finish this, please press RET."
 ŁA܂肽[!!"
   "2.Then input additional string *at CURSOR POSITION*
 According to last example \\documentstyle{},
-modify it \\documentstyle[12pt]{}.  RET to finish."
-)
+modify it \\documentstyle[12pt]{}.  RET to finish.")
 
 (YaTeX-setq YaTeX-generate-put-arg-message
   "3.̂AL[{[hǂݍŗ~ɓāB
@@ -87,8 +77,7 @@ modify it \\documentstyle[12pt]{}.  RET to finish."
   "3.In this string, extract string which you want to input from key
 board with quiry afterwards.  For example, though additional string is
 \\documentstyle[12pt]{}, but you want enter only `12pt' by hand.
-RET to finish!"
-)
+RET to finish!")
 
 (YaTeX-setq YaTeX-generate-read-prompt-message
   "4.ł́AƂł̕ǂݍގɁAǂvvg
@@ -96,8 +85,7 @@ oł? ɓĉBʓ|ȂPɃ^[łĂˁB
  12pt ̕AwTCÝxƂ߁B"
   "4.When you use this add-in function afterwards, what message
 do you like to be prompted with to enter these values.  In last
-example `12pt', typical prompt string may be `Size?: '."
-)
+example `12pt', typical prompt string may be `Size?: '.")
 
 (YaTeX-setq YaTeX-generate-done-message
   "悵! ꂪAN̍肽֐B~/.emacs ɂłĂ
@@ -110,45 +98,45 @@ this description to your ~/.emacs or so.  Use this buffer(*Guide*)
 for testing of this function please.
   But you can see this function quite easy, can't you? You had better
 write your most favorite add-in function yourself!
-"
-)
+")
 
 (YaTeX-setq YaTeX-generate-nomatch-message
   "炱Aȕǂɂˁ[!!"
-  "No such string in additional string."
-)
+  "No such string in additional string.")
+
 (YaTeX-setq YaTeX-generate-buffer
   "*t֐obt@*"
-  "*Generate-add-in-function*"
-)
+  "*Generate-add-in-function*")
 
 (YaTeX-setq YaTeX-generate-message-buffer
   "*ē*"
-  "*Guide*"
-)
+  "*Guide*")
 
 (YaTeX-setq YaTeX-generate-bug-message
   "߁`!! ƁÃAhC֐̎s݂!!
 ҂܂ŘAĂ```!"
   "Sorry I failed to make add-in function for you...
-Send bug report to me."
-)
+Send bug report to me.")
 
 (YaTeX-setq YaTeX-generate-narrow-message
   "ʂ܂悤ȋC܂B"
-  "Too narrow screen height."
-)
+  "Too narrow screen height.")
 
 (defvar YaTeX-generate-message-height
   10 "Window height of YaTeX-generate-message-buffer")
 
+;; Do you need learning for generated function?
+;; If you need, please tell me (yuuji@yatex.org)
+;;(defvar YaTeX-generate-variables-for-learning nil)
+;;(defvar YaTeX-generate-current-completion-table nil)
+
 ;;;
 ;Generate mode.
 ;;;
 (defun YaTeX-generate ()
   "Genarate YaTeX add-in function with enquiry."
   (interactive)
-  (if (< (screen-height) (+ YaTeX-generate-message-height 10))
+  (if (< (YaTeX-screen-height) (+ YaTeX-generate-message-height 10))
       (error YaTeX-generate-narrow-message))
   (put 'YaTeX-generate 'disabled t)
   (save-window-excursion
@@ -176,7 +164,7 @@ Send bug report to me."
 	  (YaTeX-suppress-sparse-keymap map)
 	  ;;First get input form.
 	  (recursive-edit)
-	  (setq input (buffer-string)
+	  (setq input (YaTeX-minibuffer-string)
 		end (1- (length input)))
 	  (if (string= "" input) (error YaTeX-generate-abort-message))
 	  (YaTeX-generate-move-to-add-in-position)
@@ -196,7 +184,7 @@ Send bug report to me."
 	  (unwind-protect
 	      (recursive-edit)
 	    (fset 'self-insert-command (symbol-function 'si)))
-	  (setq output (buffer-string))
+	  (setq output (YaTeX-minibuffer-string))
 	  (cond ((string= "" output)	(error YaTeX-generate-abort-message))
 		((string= input output)	(error YaTeX-generate-same-message))
 		((< (length output) (length input))
@@ -245,8 +233,7 @@ Send bug report to me."
   (condition-case error
       (eval-current-buffer)
     (error (insert YaTeX-generate-bug-message)))
-  (pop-to-buffer YaTeX-generate-message-buffer)
-)
+  (pop-to-buffer YaTeX-generate-message-buffer))
 
 (defun YaTeX-generate-parse-add-in (args add-in)
   "Parse add-in string and extract argument for it.
@@ -257,7 +244,8 @@ Variable add-in is referred in parent function."
 	func-name (string ""))
     ;;Phase 1. extract argument from add-in string.
     (mapcar
-     '(lambda (arg)
+     (function
+      (lambda (arg)
 	(let ((index 0) (match 0) beg end (carg (car arg)))
 	  (YaTeX-generate-display-message
 	   YaTeX-generate-read-prompt-message)
@@ -278,7 +266,7 @@ Variable add-in is referred in parent function."
 	       ((YaTeX-generate-ask-match-position)
 		(YaTeX-generate-register-match))))
 	    (setq index end))
-	  (setq i (1+ i))))
+	  (setq i (1+ i)))))
      args)
     ;;Phase 2. Generate function!!
     (setq i 0)
@@ -300,17 +288,17 @@ Variable add-in is referred in parent function."
      "(defun " func-name " ()\n"
      "  (let (")
     (mapcar
-     '(lambda (arg)
+     (function
+      (lambda (arg)
 	(insert (format "(arg%d (read-string \"%s: \"))\n"
 			i (aref prompt (1- i))))
-	(setq i (1+ i)))
+	(setq i (1+ i))))
      args)
     (delete-region (point) (progn (forward-line -1) (end-of-line) (point)))
     (insert ")\n(concat " (YaTeX-generate-lisp-quote string)
 	    ")))\n")
     (indent-region (point-min) (point) nil)
-    used)
-)
+    used))
 
 (defun YaTeX-generate-ask-match-position ()
   "Ask user whether match-position is in his expectation,
@@ -327,14 +315,12 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
   (insert "\n" (if YaTeX-japan "ɂΉĂ?"
 		 "this underlined part too?"))
   (other-window -1)
-  (y-or-n-p (if YaTeX-japan "͂Ă܂" "Is underline right"))
-)
+  (y-or-n-p (if YaTeX-japan "͂Ă܂" "Is underline right")))
 
 (defun YaTeX-generate-register-match ()
   (nconc arg (list (list beg end)))
   (let ((x beg))
-    (while (< x end) (aset used x i)(setq x (1+ x))))
-)
+    (while (< x end) (aset used x i)(setq x (1+ x)))))
 
 (defun YaTeX-generate-display-message (mes &optional bottom)
   "Display message to generation buffer."
@@ -343,8 +329,7 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
   (insert "\n\n")
   (if bottom (recenter (1- bottom)) (recenter 0))
   (insert mes)
-  (other-window -1)
-)
+  (other-window -1))
 
 (defun YaTeX-generate-move-to-add-in-position ()
   "Move cursor where add-in function should insert string."
@@ -352,7 +337,7 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
    ((eq YaTeX-current-completion-type 'begin)
     (goto-char (point-min))
     (skip-chars-forward "^{")
-    (setq env-name
+    (setq YaTeX-env-name
 	  (buffer-substring (1+ (point))
 			    (progn (skip-chars-forward "^}") (point))))
     (forward-char 1))
@@ -362,18 +347,15 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
    ((eq YaTeX-current-completion-type 'maketitle)
     (goto-char (point-max))
     (if (= (preceding-char) ? )
-	(forward-char -1)))
-   )
-)
+	(forward-char -1)))))
 
 (defun YaTeX-generate-function-name ()
   (concat
    "YaTeX:"
    (cond
-    ((eq YaTeX-current-completion-type 'begin) env-name)
-    ((eq YaTeX-current-completion-type 'section) section-name)
-    ((eq YaTeX-current-completion-type 'maketitle) single-command)))
-)
+    ((eq YaTeX-current-completion-type 'begin) YaTeX-env-name)
+    ((eq YaTeX-current-completion-type 'section) YaTeX-section-name)
+    ((eq YaTeX-current-completion-type 'maketitle) YaTeX-single-command))))
 
 (defun YaTeX-generate-lisp-quote (str)
   (let ((len (length str))(i 0) (quote ""))
@@ -384,8 +366,7 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
 	  (setq quote (concat quote "\""))
 	(setq quote (concat quote (substring str i (1+ i)))))
       (setq i (1+ i)))
-    quote)
-)
+    quote))
 
 (defun YaTeX-generate-quote-quote (str)
   (let ((len (length str))(i 0) (quote ""))
@@ -394,15 +375,13 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
 	  (setq quote (concat quote (char-to-string 127))))
       (setq quote (concat quote (substring str i (1+ i))))
       (setq i (1+ i)))
-    quote)
-)
+    quote))
 
 (defun YaTeX-suppress-sparse-keymap (map)
   (let ((i ? ))
     (while (< i 127)
       (define-key map (char-to-string i) 'undefined)
-      (setq i (1+ i))))
-)
+      (setq i (1+ i)))))
 
 ;;;
 ;; Auto-generate Function for Lispers.
@@ -419,8 +398,8 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
      ((= c ?p) 'oneof)
      ((= c ?o) 'coord)
      ;;((= c ?m) 'macro)
-     (t        'quit)))
- )
+     (t        'quit))))
+
 (defun YaTeX-generate-read-completion-table ()
   (let ((i 1) cand (cands "(") (cb (current-buffer))
 	(buf (get-buffer-create " *Candidates*")))
@@ -432,52 +411,60 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
       (while (string<
 	      ""
 	      (setq cand (read-string (format "Item[%d](RET to exit): " i))))
-	(setq cands (concat cands (format "(""%s"")\n" cand))
+	(setq cands (concat cands (format "(\"%s\")\n" cand))
 	      i (1+ i))
 	(insert cand "\n"))
       (kill-buffer buf)))
     ;;(set-buffer cb)
-    (concat cands ")"))
-)
+    (setq YaTeX-generate-current-completion-table (concat cands ")"))))
+
 (defun YaTeX-generate-corresponding-paren (left)
   (cond
    ((equal left "{") "}")
    ((equal left "[") "]")
    ((equal left "(") ")")
    ((equal left "<") ">")
-   ((equal left "\\begin{" "}"))
-   (t left))
-)
+   ((equal left "\\begin{") "}")
+   (t left)))
+
 (defun YaTeX-generate-create-read-string (&optional nth)
   (concat
-   "(read-string """
+   "(read-string \""
    (read-string (if nth (format "Prompt for argument#%d: " nth) "Prompt: "))
-   ": ""\n"
-   """" (read-string "Default: ") """"
-   ")\n")
-)
+   ": \"\n"
+   "\"" (read-string "Default: ") "\""
+   ")\n"))
+
 (defun YaTeX-generate-create-completing-read (&optional nth)
-  (concat
-   "(completing-read """
-   (read-string (if nth (format "Prompt for argument#%d: " nth) "Prompt: "))
-   ": ""\n"
-   (format "'%s\n" (YaTeX-generate-read-completion-table))
-   "nil "
-   (format "%s)" (y-or-n-p "Require match? ")))
-)
+  (prog1
+      (concat
+       "(completing-read \""
+       (read-string
+	(if nth (format "Prompt for argument#%d: " nth) "Prompt: "))
+       ": \"\n"
+       (format "'%s\n" (YaTeX-generate-read-completion-table))
+       "nil "
+       (format "%s)" (y-or-n-p "Require match? ")))
+    (if nil ;;;(y-or-n-p "Do you need learning for this completion?")
+	(setq YaTeX-generate-variables-for-learning
+	      (cons
+	       (cons (format "YaTeX-%s-%d" command (or nth 0))
+		     YaTeX-generate-current-completion-table)
+	       YaTeX-generate-variables-for-learning)))))
+
 (defun YaTeX-generate-create-read-file-name (&optional nth)
   (concat
-   "(read-file-name """
+   "(read-file-name \""
    (read-string (if nth (format "Prompt for argument#%d: " nth) "Prompt: "))
-   ": "" nil nil t """")\n")
-)
+   ": \" "" \"\" t \"\")\n"))
+
 (defun YaTeX-generate-create-read-oneof (&optional nth readpos)
   (concat
    (if readpos
-       "(YaTeX:read-position """
-     "(YaTeX:read-oneof """)
-   (read-string "Acceptable characters: " "lcr") """)\n")
-)
+       "(YaTeX:read-position \""
+     "(YaTeX:read-oneof \"")
+   (read-string "Acceptable characters: " "lcr") "\")\n"))
+
 (defun YaTeX-generate-option-type (command)
   (let ((func (format "YaTeX:%s" command)) leftp
 	(buf (get-buffer-create YaTeX-generate-buffer)) type (n 1))
@@ -491,41 +478,39 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
 	 (cond
 	  ;;Read string
 	  ((eq type 'string)
-	   (concat """" (setq leftp (read-string "Left parenthesis: " "{"))
-		   """\n"
+	   (concat "\"" (setq leftp (read-string "Left parenthesis: " "("))
+		   "\"\n"
 		   (YaTeX-generate-create-read-string)
-		   """" (YaTeX-generate-corresponding-paren leftp) """"
-		   ))
-	  
+		   "\"" (YaTeX-generate-corresponding-paren leftp) "\"")
+	   )
 	  ;;Completing-read
 	  ((eq type 'completion)
-	   (concat """" (setq leftp (read-string "Left parenthesis: " "{"))
-		   """\n"
+	   (concat "\"" (setq leftp (read-string "Left parenthesis: " "{"))
+		   "\"\n"
 		   (YaTeX-generate-create-completing-read)
-		   """" (YaTeX-generate-corresponding-paren leftp) """")
+		   "\"" (YaTeX-generate-corresponding-paren leftp) "\"")
 	   )
 	  ((eq type 'file)
-	   (concat """" (setq leftp (read-string "Left parenthesis: " "{"))
-		   """\n"
+	   (concat "\"" (setq leftp (read-string "Left parenthesis: " "("))
+		   "\"\n"
 		   (YaTeX-generate-create-read-file-name)
-		   """" (YaTeX-generate-corresponding-paren leftp) """")
+		   "\"" (YaTeX-generate-corresponding-paren leftp) "\"")
 	   )
 	  ((eq type 'oneof)
 	   (YaTeX-generate-create-read-oneof nil t)
 	   )
 	  ((eq type 'option)
-	   (concat "(let ((op (read-string """
+	   (concat "(let ((op (read-string \""
 		   (read-string "Prompt: ")
-		   ": "")))\n"
-		   "(if (string< """" op)\n"
-		   "    (concat ""["" op ""]"")\n"
-		   "  """"))\n")
+		   ": \")))\n"
+		   "(if (string< \"\" op)\n"
+		   "    (concat \"[\" op \"]\")\n"
+		   "  \"\"))\n")
 	   )
-	  
 	  ((eq type 'coord)
-	   (concat "(YaTeX:read-coordinates """
+	   (concat "(YaTeX:read-coordinates \""
 		   (read-string "Prompt for coordinates: ")
-		   ": """)
+		   ": \")\n")
 	   )
 	  ((eq type 'macro)
 	   (error "not yet supported")
@@ -536,8 +521,8 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
     (goto-char (point-min))
     (while (not (eobp)) (lisp-indent-line) (forward-line 1))
     (eval-current-buffer)
-    buf)
-)
+    buf))
+
 (defun YaTeX-generate-argument-type (command argc)
   "Create an argument-type add-in function."
   (interactive)
@@ -571,27 +556,32 @@ Referencing variables in parent function YaTeX-generate-parse-add-in."
     (YaTeX-update-table
      (if (> argc 1) (list command argc) (list command))
      'section-table 'user-section-table 'tmp-section-table)
-    buf)
-)
+    buf))
+
 (defun YaTeX-generate-simple (&optional command)
   "Simple but requiring some elisp knowledge add-in generator."
   (interactive)
+  (setq YaTeX-generate-variables-for-learning nil)
   (or command
       (setq command
 	    (completing-read
-	     (format "Making add-in function for (default %s): " section-name)
+	     (format
+	      "Making add-in function for (default %s): " YaTeX-section-name)
 	     (append
 	      section-table user-section-table tmp-section-table
-	      article-table user-article-table
 	      env-table     user-env-table     tmp-env-table
 	      singlecmd-table user-singlecmd-table tmp-singlecmd-table)
 	     nil nil)
-	    command (if (string= "" command) section-name command)))
-  (message "(o)ǉ^? (a)^? (yatexadd.docQƂ̂) :")
+	    command (if (string= "" command) YaTeX-section-name command)))
+  (message
+   (cond
+    (YaTeX-japan "(o)ǉ^? (a)^? (yatexadd.docQƂ̂) :")
+    (t "(O)ption? (A)rgument?")))
   (YaTeX-showup-buffer
    (if (= (read-char) ?o)
        (YaTeX-generate-option-type command)
      (YaTeX-generate-argument-type
       command
-      (string-to-int (read-string "How many arguments?: ")))) nil)
-)
+      (YaTeX-str2int (read-string "How many arguments?: ")))) nil))
+
+(provide 'yatexgen)
